# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['firestone', 'firestone.schema', 'firestone.spec']

package_data = \
{'': ['*']}

install_requires = \
['Jinja2>=3.1.2,<4.0.0',
 'aiohttp>=3.8.4,<4.0.0',
 'click>=8.1.3,<9.0.0',
 'firestone-lib>=0.1.0,<0.2.0',
 'jsonref>=1.0.0.post1,<2.0.0',
 'jsonschema>=4.16.0,<5.0.0',
 'pydantic>=1.10.5,<2.0.0',
 'python-dateutil>=2.8.2,<3.0.0',
 'pyyaml>=6.0,<7.0',
 'setuptools>=65.5.0,<66.0.0']

extras_require = \
{'caching': ['quart>=0.18.3,<0.19.0']}

entry_points = \
{'console_scripts': ['firestone = firestone.__main__:main']}

setup_kwargs = {
    'name': 'firestoned',
    'version': '0.2.1',
    'description': 'Build OpenAPI and AsyncAPI specs based off one or more resource json schema files',
    'long_description': '[![Last PR Build 🐍](https://github.com/firestoned/firestone/actions/workflows/pr.yml/badge.svg)](https://github.com/firestoned/firestone/actions/workflows/pr.yml)\n<!-- Pytest Coverage Comment:Begin -->\n<a href="https://github.com/firestoned/firestone/blob/main/README.md"><img alt="Coverage" src="https://img.shields.io/badge/Coverage-38%25-red.svg" /></a>\n<!-- Pytest Coverage Comment:End -->\n\n# Firestone\n\nResource-Based Approach to Building APIs\n\n``firestone`` allows you to build OpenAPI, AsyncAPI and gRPC specs based off one or\nmore resource json schema files. This allows you to focus on what really\nmatters, the resource you are developing!\n\nOnce you have generated the appropriate specification file for your project, you\ncan then use the myriad of libraries and frameworks to generate stub code for\nyou.\n\n**The primary premise of this project is not to introduce any new "language" to describe your\nresources(s), use JSON Schema!**\n\nTHis makes it easy to come up to speed and little to no prior knowledge to get\ngoing.\n\nHaving said that, the schema for a resource provides additional helpful functionality,\nsee [schema](#schema) section.\n\n## Quick Start\n\nYou can use pip or poetry to install and run ``firestone``. We suggest using pip if you wish to install\n`firestone`` machine-wide, else, for local use, use poetry.\n\n### pip\n\nIt\'s a simple as running the following ``pip`` command:\n\n```\nsudo pip install firestone\n```\n\n### poetry\n\n[Poetry](https://python-poetry.org/) is a great build tool for python that\nallows you to build and run all locally in a virtual environment. This is great\nfor checking out the tool and playing around with `firestone` itself.\n\n```\nbrew install poetry\npoetry install\npoetry build\n```\n\n## Running\n\nNow that you have a copy of ``firestone``, let\'s try running it with the\nexample resource provided, an addressbook!\n\nNote: if running within poetry build, simply prepend commands with ``poetry run``\n\nFor the remainder of this documentation, we will assume you have installed firestone.\n\n### Generate an OpenAPI Spec\n\n```\nfirestone \\\n    generate \\\n    --title \'Addressbook resource\' \\\n    --description \'A simple addressBook example\' \\\n    --resources examples/addressBook/resource.yaml \\\n    openapi\n    --security \'{"name": "bearer_auth", "scheme": "bearer", "type": "http", "bearerFormat": "JWT"}\' \\\n```\n\nLet\'s quickly dissect this command:\n\n- we are telling firestone to generate an `openapi` spec, given the ``title``,\n  ``description`` and the two given resource files.\n- By default, this will output the specification file to stdout, alternatively\n  you can provide the `-O` option to output to a specific file.\n\nYou can also, add the command line `--ui-server` tot he end, which will launch a\nsmall webserver and run the Swagger UI to view this specification file.\n\n```\nfirestone --debug generate --title \'Example person and addressBook API\' \\\n    --description \'An example API with more than one resource\' \\\n    --resources examples/addressBook.yaml,examples/person.yaml \\\n    openapi \\\n    --security \'{"name": "bearer_auth", "scheme": "bearer", "type": "http", "bearerFormat": "JWT"}\' \\\n    --ui-server\n# ...\n* Serving Quart app \'firestone.__main__\'\n* Environment: production\n* Please use an ASGI server (e.g. Hypercorn) directly in production\n* Debug mode: False\n* Running on http://127.0.0.1:5000 (CTRL + C to quit)\n[2022-10-31 02:47:17 -0500] [87590] [INFO] Running on http://127.0.0.1:5000 (CTRL + C to quit)\n# 2022-10-31 02:47:17,120 - [MainThread] hypercorn.error:102 INFO - Running on http://127.0.0.1:5000 (CTRL + C to quit)\n```\n\nNow you can use your browser to navigate to `http://127.0.0.1:5000/apidocs`\n\n## Schema\n\nIt all begins with your resource definition! This is done using JSON schema and\nwe have provided an example in our `examples` directory, called addressBook. We\nwill use this to describe how the schema is setup and how you can adapt to your\nown.\n\nHere is the full file:\n\n```yaml\nname: addressBook\ndescription: An example of an addressBook resource\nversion: 1.0\nversion_in_path: false\ndefault_query_params:\n  - name: limit\n    description: Limit the number of responses back\n    in: params\n    schema:\n      type: integer\n  - name: offset\n    description: The offset to start returning resources\n    in: params\n    schema:\n      type: integer\ndescriptions:\n  resource:\n    get: List all addresses in this addressbook.\n    head: Determine the existence and size of addresses in this addressbook.\n    patch: Patch one or more addresses in this addressbook.\n    post: Create a new address in this addressbook, a new address key will be created.\n    delete: Delete all addresses from this addressbook.\n  instance:\n    get: Get a specific address from this addressbook.\n    head: Determine the existence and size of this address.\n    patch: Patch this address in the addressbook.\n    put: Put a new address in this addressbook, with the given address key.\n    delete: Delete an address from this addressbook.\nschema:\n  type: array\n  key:\n    name: address_key\n    schema:\n      description: A unique identifier for an addressbook entry.\n      type: string\n  #responseCodes:\n  # - 200\n  # - 201\n  query_params:\n    - name: city\n      description: Filter by city name\n      schema:\n        type: string\n      methods:\n        - get\n  items:\n    type: object\n    properties:\n      addrtype:\n        description: The address type, e.g. work or home\n        type: string\n        enum:\n          - work\n          - home\n      street:\n        description: The street and civic number of this address\n        type: string\n      city:\n        description: The city of this address\n        type: string\n      state:\n        description: The state of this address\n        type: string\n      country:\n        description: The country of this address\n        type: string\n```\n\n### Metadata\n\nThere is a certain amount of metadata that all of these specifications\nuse/require, and this is done at the top of the resource,yaml; for posterity,\nthey are:\n\n```yaml\nname: addressBook\ndescription: An example of an addressBook resource\nversion: 1.0\n```\n\n#### `name`\n\nThe name is used in various places, including as the root to API URLs, for\nexample in OpenAPI, `/addressBook`\n\n#### `description`\n\nThis is self evident, I hope, the description of this resource and is used nt he\ngenerated specification files.\n\n#### `descriptions`\n\nThis is a map/dict of either `resource` and/or `instance`, which itself is a map or\nmethods to descriptions, e.g.:\n\n\n```yaml\ndescriptions:\n  resource:\n    get: List all addresses in this addressbook.\n    head: Determine the existence and size of addresses in this addressbook.\n    patch: Patch one or more addresses in this addressbook.\n    post: Create a new address in this addressbook, a new address key will be created.\n    delete: Delete all addresses from this addressbook.\n```\n\n#### `methods`\n\nThis is a map/dict of `resource`, and/or `instance`, and/or `instance_attrs`\n(the instance attributes to expose), and a list of methods to explicitly expose,\ne.g.:\n\n\n```yaml\nmethods:\n  resource:\n    - get\n    - post\n    - put\n```\n\n#### `version`\n\nThe version of this resource definition, this cna alternatively be used in the\nURL as well, see below `version_in_path`\n\n#### `version_in_path`\n\nThis attribute defines whether to prepend the version defined above in the URL paths, e.g., for the\nabove, you would get: `/v1.0/addressBook`.\n\n#### `default_query_params`\n\nYou can provide a list of default query parameters that will be added to all HTTP methods,\nor optionally you can provide a list of the HTTP methods, for which `firestone` will add.\n\n\n### Generate OpenAPI Client\n\nNow, to generate your OpenAPI client, you will need the `openapi-generator` command, and this can be\nused to generate client code in many languages. For more details on this project, see [here](https://openapi-generator.tech/).\n\nThis client code can then be used as an SDK or used by our CLI generation, see below.\n\n```\nopenapi-generator generate \\\n    -i examples/addressbook/openapi.yaml \\\n    -g python-nextgen \\\n    -o /tmp/addressbook-client \\\n    --skip-validate-spec \\\n    -c examples/addressbook/openapi-gen-config.json\n```\n\n### Generate OpenAPI Client\n\nNow that you have generated the client code, you can also generate a CRUD, Python Click-based\nCLI around your code. This generator creates a standalone script or as a module to be used\nin your console scripts, as part of your project build.\n\nHere is an example command we use to generate the example Addressbook.\n\n```\nfirestone generate \\\n    --title \'Addressbook CLI\' \\\n    --description \'This is the CLI for the example Addressbook\' \\\n    --resources examples/addressbook/addressbook.yaml,examples/addressbook/person.yaml \\\n    --version 1.0 \\\n     cli \\\n     --pkg addressbook \\\n     --client-pkg addressbook.client > examples/addressbook/main.py\n```\n\nLet\'s quickly dissect this command:\n\n- we are telling firestone to generate an `openapi` spec, given the ``title``,\n  ``description`` and the two given resource files.\n- By default, this will output the specification file to stdout, alternatively\n  you can provide the `-O` option to output to a specific file.\n\n## Contributing\n',
    'author': 'Erick Bourgeois',
    'author_email': 'erick@jeb.ca',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/firestoned/firestone',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
